/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMAXIMX2ARTWORK_HPP
#define BINARY_ZAMAXIMX2ARTWORK_HPP

namespace ZaMaximX2Artwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* ledredData;
    const unsigned int ledredDataSize = 484;
    const unsigned int ledredWidth    = 11;
    const unsigned int ledredHeight   = 11;

    extern const char* ledyellowData;
    const unsigned int ledyellowDataSize = 484;
    const unsigned int ledyellowWidth    = 11;
    const unsigned int ledyellowHeight   = 11;

    extern const char* zamaximx2Data;
    const unsigned int zamaximx2DataSize = 180081;
    const unsigned int zamaximx2Width    = 561;
    const unsigned int zamaximx2Height   = 107;
}

#endif // BINARY_ZAMAXIMX2ARTWORK_HPP

