/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMEQ2ARTWORK_HPP
#define BINARY_ZAMEQ2ARTWORK_HPP

namespace ZamEQ2Artwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* ledredData;
    const unsigned int ledredDataSize = 484;
    const unsigned int ledredWidth    = 11;
    const unsigned int ledredHeight   = 11;

    extern const char* ledyellowData;
    const unsigned int ledyellowDataSize = 484;
    const unsigned int ledyellowWidth    = 11;
    const unsigned int ledyellowHeight   = 11;

    extern const char* toggleoffData;
    const unsigned int toggleoffDataSize = 90048;
    const unsigned int toggleoffWidth    = 112;
    const unsigned int toggleoffHeight   = 201;

    extern const char* toggleonData;
    const unsigned int toggleonDataSize = 88440;
    const unsigned int toggleonWidth    = 110;
    const unsigned int toggleonHeight   = 201;

    extern const char* togglesliderData;
    const unsigned int togglesliderDataSize = 676;
    const unsigned int togglesliderWidth    = 13;
    const unsigned int togglesliderHeight   = 13;

    extern const char* zameq2Data;
    const unsigned int zameq2DataSize = 427275;
    const unsigned int zameq2Width    = 633;
    const unsigned int zameq2Height   = 225;
}

#endif // BINARY_ZAMEQ2ARTWORK_HPP

