/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMCOMPARTWORK_HPP
#define BINARY_ZAMCOMPARTWORK_HPP

namespace ZamCompArtwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* ledredData;
    const unsigned int ledredDataSize = 484;
    const unsigned int ledredWidth    = 11;
    const unsigned int ledredHeight   = 11;

    extern const char* ledyellowData;
    const unsigned int ledyellowDataSize = 484;
    const unsigned int ledyellowWidth    = 11;
    const unsigned int ledyellowHeight   = 11;

    extern const char* toggleoffData;
    const unsigned int toggleoffDataSize = 1056;
    const unsigned int toggleoffWidth    = 22;
    const unsigned int toggleoffHeight   = 12;

    extern const char* toggleonData;
    const unsigned int toggleonDataSize = 1056;
    const unsigned int toggleonWidth    = 22;
    const unsigned int toggleonHeight   = 12;

    extern const char* zamcompData;
    const unsigned int zamcompDataSize = 256800;
    const unsigned int zamcompWidth    = 800;
    const unsigned int zamcompHeight   = 107;
}

#endif // BINARY_ZAMCOMPARTWORK_HPP

