/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMSFZARTWORK_HPP
#define BINARY_ZAMSFZARTWORK_HPP

namespace ZamSFZArtwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* loadData;
    const unsigned int loadDataSize = 6864;
    const unsigned int loadWidth    = 78;
    const unsigned int loadHeight   = 22;

    extern const char* loadprData;
    const unsigned int loadprDataSize = 6864;
    const unsigned int loadprWidth    = 78;
    const unsigned int loadprHeight   = 22;

    extern const char* zamsfzData;
    const unsigned int zamsfzDataSize = 94428;
    const unsigned int zamsfzWidth    = 258;
    const unsigned int zamsfzHeight   = 122;
}

#endif // BINARY_ZAMSFZARTWORK_HPP

