/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMPIANOARTWORK_HPP
#define BINARY_ZAMPIANOARTWORK_HPP

namespace ZamPianoArtwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* zampianoData;
    const unsigned int zampianoDataSize = 189720;
    const unsigned int zampianoWidth    = 310;
    const unsigned int zampianoHeight   = 204;
}

#endif // BINARY_ZAMPIANOARTWORK_HPP

