/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMGEQ31ARTWORK_HPP
#define BINARY_ZAMGEQ31ARTWORK_HPP

namespace ZamGEQ31Artwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* slidereqData;
    const unsigned int slidereqDataSize = 1292;
    const unsigned int slidereqWidth    = 17;
    const unsigned int slidereqHeight   = 19;

    extern const char* zamgeq31Data;
    const unsigned int zamgeq31DataSize = 300900;
    const unsigned int zamgeq31Width    = 590;
    const unsigned int zamgeq31Height   = 170;
}

#endif // BINARY_ZAMGEQ31ARTWORK_HPP

