/* (Auto-generated binary data file). */

#ifndef BINARY_BLENDISH_RESOURCES_H
#define BINARY_BLENDISH_RESOURCES_H

namespace blendish_resources
{
    extern const char*  blender_icons16_png;
    const unsigned int  blender_icons16_png_size = 250706;

    extern const char*  dejavusans_ttf;
    const unsigned int  dejavusans_ttf_size = 741536;
};

#endif
