/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMSYNTHARTWORK_HPP
#define BINARY_ZAMSYNTHARTWORK_HPP

namespace ZamSynthArtwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* smoothrData;
    const unsigned int smoothrDataSize = 6864;
    const unsigned int smoothrWidth    = 78;
    const unsigned int smoothrHeight   = 22;

    extern const char* smoothyData;
    const unsigned int smoothyDataSize = 6864;
    const unsigned int smoothyWidth    = 78;
    const unsigned int smoothyHeight   = 22;

    extern const char* toggleoffData;
    const unsigned int toggleoffDataSize = 1056;
    const unsigned int toggleoffWidth    = 12;
    const unsigned int toggleoffHeight   = 22;

    extern const char* toggleonData;
    const unsigned int toggleonDataSize = 1056;
    const unsigned int toggleonWidth    = 12;
    const unsigned int toggleonHeight   = 22;

    extern const char* zamsynthData;
    const unsigned int zamsynthDataSize = 315000;
    const unsigned int zamsynthWidth    = 350;
    const unsigned int zamsynthHeight   = 300;
}

#endif // BINARY_ZAMSYNTHARTWORK_HPP

