/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMNOISEARTWORK_HPP
#define BINARY_ZAMNOISEARTWORK_HPP

namespace ZamNoiseArtwork
{
    extern const char* toggleoffData;
    const unsigned int toggleoffDataSize = 1056;
    const unsigned int toggleoffWidth    = 12;
    const unsigned int toggleoffHeight   = 22;

    extern const char* toggleonData;
    const unsigned int toggleonDataSize = 1056;
    const unsigned int toggleonWidth    = 12;
    const unsigned int toggleonHeight   = 22;

    extern const char* zamnoiseData;
    const unsigned int zamnoiseDataSize = 45264;
    const unsigned int zamnoiseWidth    = 164;
    const unsigned int zamnoiseHeight   = 92;
}

#endif // BINARY_ZAMNOISEARTWORK_HPP

