/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMCHILD670ARTWORK_HPP
#define BINARY_ZAMCHILD670ARTWORK_HPP

namespace ZamChild670Artwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* zamchild670Data;
    const unsigned int zamchild670DataSize = 151191;
    const unsigned int zamchild670Width    = 471;
    const unsigned int zamchild670Height   = 107;
}

#endif // BINARY_ZAMCHILD670ARTWORK_HPP

