/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMAUTOSATARTWORK_HPP
#define BINARY_ZAMAUTOSATARTWORK_HPP

namespace ZamAutoSatArtwork
{
    extern const char* zamautosatData;
    const unsigned int zamautosatDataSize = 59148;
    const unsigned int zamautosatWidth    = 212;
    const unsigned int zamautosatHeight   = 93;
}

#endif // BINARY_ZAMAUTOSATARTWORK_HPP

