/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMGATEARTWORK_HPP
#define BINARY_ZAMGATEARTWORK_HPP

namespace ZamGateArtwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* ledredData;
    const unsigned int ledredDataSize = 484;
    const unsigned int ledredWidth    = 11;
    const unsigned int ledredHeight   = 11;

    extern const char* ledyellowData;
    const unsigned int ledyellowDataSize = 484;
    const unsigned int ledyellowWidth    = 11;
    const unsigned int ledyellowHeight   = 11;

    extern const char* zamgateData;
    const unsigned int zamgateDataSize = 206724;
    const unsigned int zamgateWidth    = 644;
    const unsigned int zamgateHeight   = 107;
}

#endif // BINARY_ZAMGATEARTWORK_HPP

