/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMHEADX2ARTWORK_HPP
#define BINARY_ZAMHEADX2ARTWORK_HPP

namespace ZamHeadX2Artwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* zamheadx2Data;
    const unsigned int zamheadx2DataSize = 228084;
    const unsigned int zamheadx2Width    = 332;
    const unsigned int zamheadx2Height   = 229;
}

#endif // BINARY_ZAMHEADX2ARTWORK_HPP

