/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMULTICOMPX2ARTWORK_HPP
#define BINARY_ZAMULTICOMPX2ARTWORK_HPP

namespace ZaMultiCompX2Artwork
{
    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* ledredData;
    const unsigned int ledredDataSize = 484;
    const unsigned int ledredWidth    = 11;
    const unsigned int ledredHeight   = 11;

    extern const char* ledyellowData;
    const unsigned int ledyellowDataSize = 484;
    const unsigned int ledyellowWidth    = 11;
    const unsigned int ledyellowHeight   = 11;

    extern const char* toggleoffData;
    const unsigned int toggleoffDataSize = 1056;
    const unsigned int toggleoffWidth    = 12;
    const unsigned int toggleoffHeight   = 22;

    extern const char* toggleoffhorizData;
    const unsigned int toggleoffhorizDataSize = 1056;
    const unsigned int toggleoffhorizWidth    = 22;
    const unsigned int toggleoffhorizHeight   = 12;

    extern const char* toggleonData;
    const unsigned int toggleonDataSize = 1056;
    const unsigned int toggleonWidth    = 12;
    const unsigned int toggleonHeight   = 22;

    extern const char* toggleonhorizData;
    const unsigned int toggleonhorizDataSize = 1056;
    const unsigned int toggleonhorizWidth    = 22;
    const unsigned int toggleonhorizHeight   = 12;

    extern const char* togglesliderData;
    const unsigned int togglesliderDataSize = 676;
    const unsigned int togglesliderWidth    = 13;
    const unsigned int togglesliderHeight   = 13;

    extern const char* zamulticompx2Data;
    const unsigned int zamulticompx2DataSize = 714630;
    const unsigned int zamulticompx2Width    = 581;
    const unsigned int zamulticompx2Height   = 410;
}

#endif // BINARY_ZAMULTICOMPX2ARTWORK_HPP

