/* (Auto-generated binary data file). */

#ifndef BINARY_ZAMTUBEARTWORK_HPP
#define BINARY_ZAMTUBEARTWORK_HPP

namespace ZamTubeArtwork
{
    extern const char* insaneoffData;
    const unsigned int insaneoffDataSize = 7104;
    const unsigned int insaneoffWidth    = 74;
    const unsigned int insaneoffHeight   = 24;

    extern const char* insaneonData;
    const unsigned int insaneonDataSize = 7104;
    const unsigned int insaneonWidth    = 74;
    const unsigned int insaneonHeight   = 24;

    extern const char* knobData;
    const unsigned int knobDataSize = 7056;
    const unsigned int knobWidth    = 42;
    const unsigned int knobHeight   = 42;

    extern const char* notchData;
    const unsigned int notchDataSize = 2016;
    const unsigned int notchWidth    = 21;
    const unsigned int notchHeight   = 24;

    extern const char* zamtubeData;
    const unsigned int zamtubeDataSize = 423360;
    const unsigned int zamtubeWidth    = 448;
    const unsigned int zamtubeHeight   = 315;
}

#endif // BINARY_ZAMTUBEARTWORK_HPP

